/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#ifndef DXC_CALLBACK_HPP
#define DXC_CALLBACK_HPP

#ifdef _WIN32
#include <Pthreads-w32/pthread.h>
#else
#include <pthread.h>
#endif

#include <DxcDefs.hpp>
#include <DataTypes/DxcData.hpp>

namespace Dxc
{
/**
*  @brief The abstract definition of the DcxData callback class.
*
*  This abstract class forms the base class for the DXC data callback.
*  Users must extend this abstract class to give their own callback
*  handler, and register the class with a DxcConnector via a ConnectorFactory.
*  Only one callback
*  can be registered per connector, but the same callback can be registered
*  with different connectors. When new data is received by a connector,
*  the handler function will be called in the context of
*  a new thread. The DxcConnector stores a reference to the instance of
*  the callback class; users are responsible for ensuring that the instance
*  is not destroyed before the connector stops receiving data.
*/
class DXCAPI_EXPORT DxcCallback
{
public:
    // Synchronized callback method.
    // Do not override this method unless you know what you're doing.
    virtual void onDataReceived(const DxcData *);

    /**
     * @brief Method to be called when data is received.
     *
     * Implement this method to receive DxcData messages. Each message
     * must be downcast into its specific type upon arrival to access
     * methods belonging to instances of DxcData subclasses.
     *
     * @param data - the message to be processed by the callback function
     */
    virtual void processData(const DxcData *data) = 0;

    DxcCallback();
    virtual ~DxcCallback();
private:
    pthread_mutex_t cbMutex_;
};
}

#endif
