/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia
  
  The DXC Framework is free software: you can redistribute it and/or modify it 
  under the terms of the GNU Lesser General Public License as published by the 
  Free Software Foundation, either version 3 of the License, or (at your 
  option) any later version. 

  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
  details. 

  You should have received a copy of the GNU Lesser General Public License 
  along with this program. If not, see <http://www.gnu.org/licenses/> 
 */
package org.dxc.api.datatypes;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;

/**
 * The root of the Dxc data definition hierarchy.
 */
public abstract class DxcData {

    private long timestamp;
    private String source;

    /**
     * Construct with current time as timestamp
     * 
     */
    public DxcData() {
        this.timestamp = System.currentTimeMillis();
        this.source = null;
    }

    /**
     * Constructor
     * 
     * @param timestamp
     *            - the desired timestamp in milliseconds since 1970-1-1
     *            00:00:00 GMT
     */
    public DxcData(long timestamp) {
        this.timestamp = timestamp;
        this.source = null;
    }

    /**
     * Get message timestamp
     * 
     * @return The message timestamp in milliseconds since 1970-1-1 00:00:00 GMT
     */
    public long getTimeStamp() {
        return timestamp;
    }

    /**
     * Set timestamp for DxcData
     * 
     * Although timestamps can be freely manipulated by users in local instances
     * of DxcData, they serve only for informative purposes. There is no
     * guarantee that a DxcData message recipient won't overwrite the timestamp
     * with the time of message arrival.
     * 
     * @param timestamp
     *            - The desired timestamp in milliseconds since 1970-1-1
     *            00:00:00 GMT
     */
    public void setTimeStamp(long timestamp) {
        this.timestamp = timestamp;
    }

    /**
     * Get message source
     * 
     * @return Source of message
     */
    public String getSource() {
        return source;
    }

    /**
     * Set source for DxcData
     * 
     * Although this field can be freely manipulated by users in local instances
     * of DxcData, they serve only for informative purposes. There is no
     * guarantee that a DxcData message source won't be overwritten after it's
     * sent.
     * 
     * @param source
     *            - source of message
     */
    public void setSource(String source) {
        this.source = source;
    }

    /**
     * Prints DxcData.
     */
    public abstract OutputStream put(OutputStream out);

    /**
     * Returns String representation of DxcData.
     */
    public String toString() {
        OutputStream bos = new ByteArrayOutputStream();
        put(bos);
        return bos.toString();
    }

    /**
     * Static method to obtain current time.
     * 
     * @return The current time in milliseconds since 1970-1-1 00:00:00 GMT
     */
    public static long getCurrentTime() {
        return System.currentTimeMillis();
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((source == null) ? 0 : source.hashCode());
        result = prime * result + (int) (timestamp ^ (timestamp >>> 32));
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        DxcData other = (DxcData) obj;
        if (source == null) {
            if (other.source != null)
                return false;
        } else if (!source.equals(other.source))
            return false;
        if (timestamp != other.timestamp)
            return false;
        return true;
    }
}