/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia
  
  The DXC Framework is free software: you can redistribute it and/or modify it 
  under the terms of the GNU Lesser General Public License as published by the 
  Free Software Foundation, either version 3 of the License, or (at your 
  option) any later version. 

  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
  details. 

  You should have received a copy of the GNU Lesser General Public License 
  along with this program. If not, see <http://www.gnu.org/licenses/> 
 */
package org.dxc.api.datatypes;

import java.io.OutputStream;
import java.io.PrintStream;

/**
 * Defines the data structure for fault injection information message.
 * 
 * During an experiment, fault injection information is sent to the apropriate
 * parties to enable benchmarking of diagnosis algorithms. This information will
 * be inaccessible by diagnosers during competition scenarios. Unlike
 * DiagnosisData, FaultInjectData is used to communicate an *actual* faulty
 * component and mode.
 */
public class FaultInjectData extends DxcData {
    private Fault f;

    private boolean isInjection;

    /**
     * Constructor.
     * 
     * @param timestamp
     *            - message time.
     * @param fault
     *            - the fault being injected/removed
     * @param isInjection
     *            - was the fault injected (true), or removed (false)?
     */
    public FaultInjectData(long timestamp, Fault fault, boolean isInjection) {
        super(timestamp);
        this.isInjection = isInjection;
        this.f = fault;
    }

    /**
     * Constructor.
     * 
     * @param fault
     *            - the fault being injected/removed
     * @param isInjection
     *            - was the fault injected (true), or removed (false)?
     */
    public FaultInjectData(Fault fault, boolean isInjection) {
        super(DxcData.getCurrentTime());
        this.f = fault;
        this.isInjection = isInjection;
    }

    public Fault getFault() {
        return f;
    }

    /**
     * True if the fault in question is being injected, false if it's being
     * stopped.
     */
    public boolean isInjection() {
        return isInjection;
    }

    /**
     * Set to true if the fault is being injected, false if it's being stopped.
     */
    public void isInjection(boolean isInjection) {
        this.isInjection = isInjection;
    }

    public OutputStream put(OutputStream os) {
        PrintStream out = new PrintStream(os);
        out.append("faultInjection @");
        out.append(Long.toString(this.getTimeStamp()));
        if (this.getSource() != null)
            out.append(" <" + this.getSource() + ">");
        if (isInjection)
            out.append(" isInjection = true, ");
        else
            out.append(" isInjection = false, ");

        out.append("fault = { " + f.toString() + "};\n");

        return out;
    }

    public boolean equals(Object o) {
        if (o == this)
            return true;
        if ((o == null) || (o.getClass() != this.getClass()))
            return false;

        FaultInjectData fid = (FaultInjectData) o;

        if ((fid.getTimeStamp() != getTimeStamp())
                || (!fid.getSource().equals(getSource()))
                || (fid.isInjection != isInjection)
                || (!fid.getFault().equals(this.getFault()))
                || (!fid.getFault().getParameterMap().equals(
                        this.getFault().getParameterMap())))
            return false;

        return true;
    }

    public int hashCode() {
        int hash = (int) getTimeStamp() + (isInjection ? 103 : 53);
        hash = 31 * hash + getSource().hashCode();
        hash = 31 * hash + f.hashCode();
        hash = 31 * hash + f.getParameterMap().hashCode();
        return hash;
    }
}
