package org.dxc.api.parser;

import static org.junit.Assert.*;

import java.io.StringReader;

import org.junit.Test;
import org.dxc.api.parser.DxcParser;
import org.dxc.api.parser.ParseException;
import org.dxc.api.parser.TokenMgrError;
import org.dxc.api.DxcCallback;
import org.dxc.api.DxcDataGenerator;
import org.dxc.api.datatypes.*;

public class TestParser {
    private class MessageHandler extends DxcCallback {
        private DxcData d;

        public MessageHandler() {
            d = null;
        }

        public void processData(DxcData d) {
            if (this.d != null)
                fail("MessageHandler received more than one message.");
            else
                this.d = d;
        }

        public DxcData getMessage() {
            if (d == null) {
                fail("MessageHandler received no message.");
                return null;
            } else {
                return d;
            }
        }
    }

    @Test
    public void diagnosisData() throws ParseException, Exception {
        Candidate c1 = new Candidate(), c2 = new Candidate();
        c1.getFaultSet().add(new Fault("component1", "stuck"));
        c1.getFaultSet().add(new Fault("component2", "stuck"));
        c1.setWeight(0.6);

        c2.getFaultSet().add(new Fault("component3", "offset"));
        c2.getFaultSet().add(new Fault("component2", "stuck_closed"));
        c2.setWeight(0.4);

        CandidateSet cs = new CandidateSet();
        cs.add(c1);
        cs.add(c2);

        DiagnosisData dd = new DiagnosisData(true, true, cs, "notes");
        dd.setSource("JUnitTest");
        assertTrue(dxcDataParse(dd));
    }

    @Test
    public void sensorData() throws ParseException, Exception {
        StringValueMap sensors = new StringValueMap();
        sensors.put("sensor.1", new RealValue(1219237.192673));
        sensors.put("sensor_2", new StringValue("foo. bar 123_99"));
        sensors.put("sensor3", new BoolValue(true));
        sensors.put("sensor...4", new IntegerValue(192673));

        SensorData sd = new SensorData(sensors);
        sd.setSource("JUnitTest");

        assertTrue(dxcDataParse(sd));
    }

    @Test
    public void commandData() throws ParseException, Exception {
        CommandData cd = new CommandData("testcommand", new RealValue(
                1012387.14789));

        cd.setSource("JUnitTest");

        assertTrue(dxcDataParse(cd));

        CommandSet commands = new CommandSet();
        commands.add(new Command("act.1", Value.v(1219237.192673)));
        commands.add(new Command("act_2", Value.v("foo. bar 123_99")));
        commands.add(new Command("act", Value.v("NOP")));
        commands.add(new Command("act...4", Value.v(192673)));
        cd = new CommandData(0, commands);

        assertTrue(dxcDataParse(cd));
    }

    @Test
    public void recoveryData() throws ParseException, Exception {

        CommandSet commands = new CommandSet();
        commands.add(new Command("act.1", Value.v(1219237.192673)));
        commands.add(new Command("act_2", Value.v("foo. bar 123_99")));
        commands.add(new Command("act", Value.v("NOP")));
        commands.add(new Command("act...4", Value.v(192673)));
        FaultSet fs = new FaultSet();
        fs.add(new Fault("fault1", "mode1"));
        fs.add(new Fault("fault2", "mode2"));
        RecoveryData rd = new RecoveryData(fs, commands, 400, "foobar");

        rd.setSource("JUnitTest");

        assertTrue(dxcDataParse(rd));
    }

    @Test
    public void errorData() throws ParseException, Exception {
        ErrorData ed = new ErrorData("error'.._1");
        ed.setSource("JUnitTest");

        assertTrue(dxcDataParse(ed));
    }

    @Test
    public void faultInjectData() throws ParseException, Exception {
        StringValueMap pm = new StringValueMap();
        pm.put("slope", new RealValue(12937.12738));
        pm.put("intermittent", new BoolValue(true));
        FaultInjectData fid = new FaultInjectData(new Fault("component..1",
                "c_faulty1287", pm), true);
        fid.setSource("JUnitTest");

        assertTrue(dxcDataParse(fid));
    }

    @Test
    public void profilingData() throws ParseException, Exception {
        ProfilingData pd = new ProfilingData(120973017510952L, 989275092131823L);
        pd.setSource("JUnitTest");

        assertTrue(dxcDataParse(pd));
    }

    @Test
    public void scenarioStatusData() throws ParseException, Exception {
        ScenarioStatusData ssd = new ScenarioStatusData(
                ScenarioStatusData.SDS_AWAITING_DA);
        ssd.setSource("JUnitTest");

        assertTrue(dxcDataParse(ssd));
    }

    @Test
    public void scenarioData() throws ParseException, Exception {
        ScenarioData sd = new ScenarioData("testScenario", "testSystem", "1.0",
                "testDA", "0.1", "1.0", 340000, "");
        sd.setSource("JUnitTest");

        assertTrue(dxcDataParse(sd));
    }

    @Test(expected = ParseException.class)
    public void singleQuote() throws ParseException, Exception {
        DxcData d = new ErrorData("'");
        assertTrue(dxcDataParse(d));
    }

    @Test(expected = ParseException.class)
    public void nameStartsWithNumber() throws ParseException, Exception {
        DxcData d = new CommandData("1component", new BoolValue(true));
        assertTrue(dxcDataParse(d));
    }

    @Test(expected = TokenMgrError.class)
    public void negativeTimestamp() throws ParseException, Exception {
        ErrorData d = new ErrorData(-1, "foo");
        assertTrue(dxcDataParse(d));
    }

    @Test(expected = ParseException.class)
    public void missingBrace() throws ParseException, Exception {

        StringValueMap sensors = new StringValueMap();
        sensors.put("sensor.1", new RealValue(1219237.192673));
        sensors.put("sensor_2", new StringValue("foo. bar 123_99"));

        DxcData d = new SensorData(sensors);
        String s = d.toString();
        s = s.replaceFirst("\\{", "");

        parseString(s);
    }

    @Test(expected = ParseException.class)
    public void missingComma() throws ParseException, Exception {

        StringValueMap sensors = new StringValueMap();
        sensors.put("sensor.11", new RealValue(1219237.192673));
        sensors.put("sensor_22", new StringValue("foo. bar 96_6666"));

        DxcData d = new SensorData(sensors);
        String s = d.toString();
        s = s.replaceFirst(",", " ");

        parseString(s);
    }

    @Test(expected = ParseException.class)
    public void recursiveMessage() throws ParseException, Exception {

        StringValueMap sensors = new StringValueMap();
        sensors.put("sensor.1", new RealValue(1219237.192673));
        sensors.put("sensor_2", new StringValue("foo. bar 123_99"));

        DxcData d = new SensorData(sensors);
        String s = d.toString();
        sensors.put("errorData", new StringValue(s));
        d = new SensorData(sensors);

        assertTrue(dxcDataParse(d));
    }

    private boolean dxcDataParse(DxcData d) throws ParseException, Exception {
        StringReader sr = new StringReader(d.toString());
        MessageHandler mh = new MessageHandler();
        DxcDataGenerator p = new DxcParser(sr);
        p.begin(mh);
        return mh.getMessage().equals(d);
    }

    private void parseString(String s) throws ParseException, Exception {
        StringReader sr = new StringReader(s);
        MessageHandler mh = new MessageHandler();
        DxcDataGenerator p = new DxcParser(sr);
        p.begin(mh);
    }
}