/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/

/**
 * ExampleRecoveryDA demonstrates how a diagnosis algorithm should
 * query the oracle for recovery actions. See ExampleDA for a more
 * general example.
 */

#ifdef _WIN32
#include <Windows.h>
#include <Pthreads-w32/pthread.h>
#define pthreadsInit pthread_win32_process_attach_np
#define pthreadsStop pthread_win32_process_detach_np
#else
#include <unistd.h>
#include <pthread.h>
#define pthreadsInit void
#define pthreadsStop void
#endif

#include <iostream>

#include <DxcCallback.hpp>
#include <DxcDataTypes.hpp>
#include <Connection/ConnectorFactory.hpp>

using namespace Dxc;
using namespace std;

static DxcConnector *connector;
static const RecoveryData *oracleResponse;
static pthread_cond_t oracleResponseCond;
static pthread_mutex_t oracleResponseMut;

static bool run = true;

class ExampleCallback : public DxcCallback
{
private:
    virtual void processData(const DxcData * msg)
    {

        if (string(typeid(*msg).name()) ==
                string(typeid(ScenarioStatusData).name()))
        {
            // check for scenario end, exit if ended
            const ScenarioStatusData* stat =
                dynamic_cast<const ScenarioStatusData *>(msg);

            if (stat->getStatus() == ScenarioStatusData::SDS_ENDED)
            {
                run = false;
            }
        }
        else if (string(typeid(*msg).name()) ==
                 string(typeid(RecoveryData).name()))
        {
            pthread_mutex_lock(&oracleResponseMut);
            oracleResponse =
                dynamic_cast<const RecoveryData *>(msg->clone());

            pthread_cond_broadcast(&oracleResponseCond);
            pthread_mutex_unlock(&oracleResponseMut);
        }
    };
};

int main(int argc, char **argv)
{
    ExampleCallback callback;
    pthreadsInit();
    pthread_mutex_init(&oracleResponseMut, NULL);
    pthread_cond_init(&oracleResponseCond, NULL);

    try
    {
        connector = ConnectorFactory::getDAConnector(&callback);
    }
    catch (...)
    {
        cerr << "DA unable to connect. Make sure ScenarioLoader " <<
             "(or StandaloneSDS) is running." << endl;
        return 1;
    }

    // signal to the system that we're prepared to receive data
    ScenarioStatusData ssd(ScenarioStatusData::DA_READY);
    connector->sendMessage(&ssd);

    while (run)
    {
        msleep(100);
    }

    // the scenario has ended. Now assume we have two
    // candidates in our diagnosis, each with the same
    // probability--one is a load fault, the other is
    // the empty set (system is nominal):

    FaultSet fs;
    Fault::ParameterMap fpm;

    fpm["Offset"] = new RealValue(-2.5);
    fpm["FaultTime"] = new RealValue(100); // seconds since scenario start
    fs.insert(Fault("DC485", "ResistanceOffset", fpm));

    // the first query will return what the appropriate recovery action is
    // for the load fault, and the associated cost.
    RecoveryData rd1(fs);

    CommandSet cmd;
    cmd.insert(Command("systemAction", Value::v("NOP"))); // no action

    // the second query will tell us the cost of taking no action if the
    // system is in the specified faulty state.
    RecoveryData rd2(fs, cmd);

    // critical region, lock mutex and wait for oracle to respond
    pthread_mutex_lock(&oracleResponseMut);
    connector->sendMessage(&rd1);
    pthread_cond_wait(&oracleResponseCond, &oracleResponseMut);
    const RecoveryData* response1 = oracleResponse;

    connector->sendMessage(&rd2);
    pthread_cond_wait(&oracleResponseCond, &oracleResponseMut);
    const RecoveryData* response2 = oracleResponse;

    pthread_mutex_unlock(&oracleResponseMut);

    // now, since both candidates have equal probability, a
    // simple strategy is to pick the cheapest recovery action.
    if (response1 == NULL || response2 == NULL)
    {
        cout << "ExampleDA received no RecoveryData!" << endl;
    }
    else if (response1->getCost() < 0 || response2->getCost() < 0)
    {
        cout << "ExampleDA received bad RecoveryData!" <<
             response1 << endl;
    }
    else
    {
        CommandData cmd;
        if (response1->getCost() < response2->getCost())
            cmd.setCommands(*response1->getCommands());
        else
            cmd.setCommands(*response2->getCommands());

        connector->sendMessage(&cmd);
    }

    // clean up after yourself!
    delete connector;
    delete oracleResponse, response1, response2;

    return 0;
}
